@@ -26,18 +26,12 @@
 .onAttach <- function(libname, pkgname) {
 
     ## get package version
-    vers <- library(help = "mboost")$info[[1]]
-    vers <- vers[grep("Version", vers)]
-    vers <- gsub("(Version.* )([0-9]+\\.[0-9]+-[0-9]+)", "\\2", vers)
+    vers <- packageDescription("mboost")[["Version"]]
 
-    packageStartupMessage(paste("This is mboost ", vers, ". ",
-                                "See ", sQuote("package?mboost"), " and the NEWS file\n",
-                                "for a complete list of changes.\n",
-                                "Note: The default for the computation",
-                                " of the degrees of freedom has changed.\n",
-                                "      For details see section ",
-                                sQuote("Global Options"), " of ",
-                                sQuote("?bols"), ".", sep =""))
+    packageStartupMessage("This is mboost ", vers, ". ", "See ",
+                          sQuote("package?mboost"), " and the NEWS file\n",
+                          "for a complete list of changes.\n",
+                          appendLF = TRUE)
     return(TRUE)
 }
 
